﻿using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Threading;

namespace PKap7_P14
{
    /// <summary>
    /// Klasse für Haus der Nikolaus
    /// </summary>
    class Nikolaus
    {
        private int[][] a;        // Adjazenzmatrix
        private int[] b;          // Lösungsvektor 
        private int sol;
        private String s;
        private ListBox lb;
        private Pen pen1;
        private Point[] ecken;

        /// <summary>
        /// Klasse für Nikolaushaus
        /// </summary>
        /// <param name="lb">Listbox für Ausgabe der Lösungen</param>
        /// <param name="xs">x-Koordinate 1. Ecke</param>
        /// <param name="ys">y-Koordinate 1. Ecke</param>
        /// <param name="d">Hausbreite</param>
        public Nikolaus(ListBox lb,int xs, int ys, int d)
        {
            // Adjazenzmatrix initialisieren
            a = new int[5][]
            { 
                  new int[] {0,1,1,0,1},
                  new int[] {1,0,1,0,1},
                  new int[] {1,1,0,1,1},
                  new int[] {0,0,1,0,1},
                  new int[] {1,1,1,1,0}
            };

            b = new int[9];    // Lösungsvektor
            sol = 0;           // Anzahl Lösungen 
            s = String.Empty;  // für Umwandlung Lösungsvektor nach String
            this.lb = lb;      // Ausgabe umgewandelte Lösungsvektoren

            // Hilfvariable zum zeichnen
            pen1 = new Pen(Color.Blue, 3);       // Stift mit Pfeil
            pen1.EndCap = LineCap.ArrowAnchor;
            ecken = new Point[5];                // die 5 Ecken des Hauses
            ecken[0] = new Point(xs, ys);
            ecken[1] = new Point(xs + d, ys);
            ecken[2] = new Point(xs+d,ys-d);
            ecken[3] = new Point(xs + d / 2, ys - d - d );
            ecken[4] = new Point(xs, ys - d);    
        }

        /// <summary>
        /// Destruktor
        /// </summary>
        ~Nikolaus()
        {
            pen1.Dispose();
        }

        /// <summary>
        /// Backtracking Algorithmus
        /// </summary>
        /// <param name="k"></param>
        public void back(int k)
        {            
            if ( k == 9 )
            {
                WriteSol();
            }
            else
                for ( int i = 0; i < 5; i++ )
                {
                    if ( a[i][b[k-1]] == 1 && i != b[k-1] )
                    {
                        b[k] = i;
                        a[i][b[k - 1]] = 0;
                        a[b[k - 1]][i] = 0;
                        back(k + 1);
                        a[i][b[k - 1]] = 1;
                        a[b[k - 1]][i] = 1;
                    }
                }
        }

        /// <summary>
        /// Lösungsvektor nach Listbox ausgeben
        /// </summary>
        private void WriteSol()
        {
            sol++;
            s = "";
            for ( int i = 0; i < 9; i++ )
            {
                s = s + (b[i] + 1).ToString();
                s = s + " ";
            }          
            lb.Items.Add(s);
        }

        /// <summary>
        /// Nikolaus zeichnen
        /// </summary>
        /// <param name="gc"></param>
        /// <param name="selected"></param>
        public void DrawNikolaus(Graphics gc, string selected)
        {
            // Graphics-Kontext clearen
            gc.Clear(Color.White);            

            // Eckpunkte beschriften
            Font font = new Font("Arial", 10, FontStyle.Regular);   
            for ( int i = 1; i <= 5; i++ )
            {
                gc.DrawString(i.ToString(), font, Brushes.Red, ecken[i-1]);              
            }
            Application.DoEvents();

            gc.DrawString(selected, font, Brushes.Black, 0, 0);

            string[] ecknrn = selected.Split(new Char[] {' '});
            // gc.DrawString(ecknrn[0].ToString(), font, Brushes.Red, new Point(0, 12));
            for ( int k = 0; k < 8; k++ )
            {
                Application.DoEvents();
                int i = Convert.ToInt32(ecknrn[k]) - 1;
                int j = Convert.ToInt32(ecknrn[k + 1]) - 1;
                gc.DrawLine(pen1, ecken[i], ecken[j]);
                // gc.DrawString(ecknrn[k+1].ToString(), font, Brushes.Red, new Point(0 + 11 * (k+1), 12));
                Thread.Sleep(2000);
            }   
        }
    }
}
